﻿using System;

namespace FunctionalCLR.Core.Function.Arithmetics {

    /// <summary>
    /// a = 1 + 3;
    /// echo(a);
    /// # result is 4
    /// </summary>
    public class AddNode : OperatorNode {
        public AddNode() {
            Value = "+";
            Precedence = 1;
        }

        public override object Accept(IVisitor visitor) {
            return visitor.Visit(this);
        }

        public override FuncNode GetReversedFunction(int index) {
            return new SubNode();
        }

        public override object Evaluate() {
            object l = Left.Evaluate();
            object r = Right.Evaluate();

            if (Utils.IsNumeric(l) && Utils.IsNumeric(r)) {
                double lr = Convert.ToDouble(l);
                double rr = Convert.ToDouble(r);

                return (lr + rr).ToString();
            } else if (l is string || r is string) {
                string lr = Convert.ToString(l);
                string rr = Convert.ToString(r);

                return lr + rr;
            }

            return null;
        }
    }
}
