﻿using System;
using System.Collections.Generic;
using System.Text;
using NUnit.Framework;
using FunctionalCLR;
using FunctionalCLR.Core;

namespace Zove.Test
{
    [TestFixture]
    public class LexerTests
    {
        [Test]
        public void TestComment()
        {
            Lexer l = new Lexer("hello world; #hello world\n\rthis is apple;");

            Assert.AreEqual("hello", l.NextToken().Value);
            Assert.AreEqual("world", l.NextToken().Value);
            Assert.AreEqual(";", l.NextToken().Value);

            Assert.AreEqual("this", l.NextToken().Value);
            Assert.AreEqual("is", l.NextToken().Value);
            Assert.AreEqual("apple", l.NextToken().Value);
            Assert.AreEqual(";", l.NextToken().Value);
        }

        [Test]
        public void TestNextLine()
        {
            Lexer l = new Lexer("class test\n\r{\n\rhello world;\n\r#this is a comment;\n\rfactor a, b;\n\r}");
            Assert.AreEqual("class test", l.NextLine());
            Assert.AreEqual("{", l.NextLine());
            Assert.AreEqual("hello world;", l.NextLine());
            Assert.AreEqual("#this is a comment;", l.NextLine());
        }

        [Test]
        public void TestReadUntil()
        {
            Lexer l = new Lexer("class test\n\r{\n\rhello world;\n\r#this is a comment;\n\rfactor a, b;\n\r}");

            Assert.AreEqual("class test", l.ReadUntil('\r', '\n'));
        }

        [Test]
        public void TestPeekChar()
        {
            Lexer l = new Lexer("class test\n\r{\n\rhello world;\n\r#this is a comment;\n\rfactor a, b;\n\r}");

            Assert.AreEqual('c', l.PeekChar());
        }

        [Test]
        public void TestSkipWhiteSpace()
        {
            Lexer l = new Lexer("\n\r class\n\thello");

            l.SkipWhiteSpace();
            Assert.AreEqual(3, l.Index);
        }

        [Test]
        public void TestNextTokenVerified()
        {
            Lexer l = new Lexer("hello world");

            Assert.AreEqual("hello", l.NextToken("hello").Value);
            Assert.AreEqual("world", l.NextToken("world").Value);
        }
        [Test]
        public void TestNextToken()
        {
            Lexer lexer = new Lexer("a + b - 100");

            Assert.AreEqual("a", lexer.NextToken().Value);
            Assert.AreEqual("+", lexer.NextToken().Value);
            Assert.AreEqual("b", lexer.NextToken().Value);
            Assert.AreEqual("-", lexer.NextToken().Value);
            Assert.AreEqual("100", lexer.NextToken().Value);
        }

        [Test]
        public void TestHasMoreTokens()
        {
            Lexer l = new Lexer("hell world;~!@$%^&*?(){};'/|,_"); //[]:\\.<>

            Assert.IsTrue(l.HasMoreTokens());
            while (l.HasMoreTokens())
            {
                l.NextToken();
            }
        }

        [Test]
        public void TestConstructor()
        {
            try
            {
                Lexer l = new Lexer(string.Empty);
                Assert.IsTrue(false);
            }
            catch
            {
                Assert.IsTrue(true);
            }
        }
    }
}
