﻿using System;
using System.Collections.Generic;
using System.IO;

namespace EmbeddedSharp.Core {
    public class DefaultConstantManager : IConstantManager {

        /// <summary>
        /// Infinity constant
        /// </summary>
        private const string CONST_INFI = "infinity";

        /// <summary>
        /// Infinity constant
        /// </summary>
        private const string CONST_EPSILON = "epsilon";

        private Dictionary<string, string> constants;

        public DefaultConstantManager(string fileName) {
            this.constants = new Dictionary<string, string>();

            if (!File.Exists(fileName)) {
                return;
            }
          
            FileStream fs = new FileStream(fileName, FileMode.Open);
            StreamReader reader = new StreamReader(fs);

            string ln = string.Empty;
            while ((ln = reader.ReadLine()) != null) {
                string name = ln.Split('=')[0];
                string value = ln.Split('=')[1];

                constants[name] = value;
            }

            reader.Close();
        }
        #region IConstantManager Members

        public bool IsConstant(string name) {
            if (name == CONST_INFI) {
                return true;
            }

            return this.constants.ContainsKey(name);
        }

        public string GetConstant(string name) {
            if (name == CONST_INFI) {
                return Double.PositiveInfinity.ToString();
            } else if (name == CONST_EPSILON) {
                return Double.Epsilon.ToString();
            }

            return this.constants[name];
        }

        #endregion
    }
}
