﻿using System;
using EmbeddedSharp.Core;
using Zove.Runtime.Core;
using Zove.Runtime.Core.Zoom;

namespace Zove.Scripting.Core.Functions {
    /// <summary>
    /// Populate an instance for a particular class.
    /// </summary>
    [Function("populate", "Populate an instance for a particular class.",
        "c=populate(\"circle\");\n\r", "isInstanceOf(c, \"circle\")")]
    public class PopulateNode : ZoveScriptingContextFuncNode {

        /// <summary>
        /// Default constructor
        /// </summary>
        public PopulateNode()
            : base("populate") {
        }

        public override object Evaluate() {
            if (Parameters.Count < 2) {
                throw new ArgumentException(
                    "usage: populate(modelName)");
            }

            string modelName = Convert.ToString(
                Parameters[1].Evaluate());
            if (string.IsNullOrEmpty(modelName)) {
                throw new ArgumentException(
                    "usage: populate(modelName)");
            }


            string instanceName = Convert.ToString(Parameters[0].Evaluate());
            try {
                ZoomLanguageNature languageNature = new ZoomLanguageNature();
                ClassContentProvider contentProvider = new ClassContentProvider(modelName, languageNature);
                ZoveObject instance = new ZoveObject(
                    instanceName, contentProvider, languageNature);
                return instance;
            } catch (Exception ex) {
                Utils.Log(ex.Message);
            }

            return null;
        }
    }
}
